        PAGE    60,132
NAME    CMOSSAVE
        TITLE   CMOSSave Save CMOS to a file on disk or floppy
Comment |
        Version 4.5 by Roedy Green
        works with MASM 6.0 and Optasm

Note that we never compute or verify a checksum.  Every vendor computes
may compute it a slightly different way.

To come:
  /V:xx - additional volatile byte

See CMOS.TXT for details on use.

  USAGE:

Examples:
*********

CMOSSave A:\MyCMOS.Sav /Q

CMOSRest A:\MyCMOS.Sav /Q

CMOSChk  A:\ByCMOS.Sav /Q

These are DOS utilities and thus can only use short 8.3 file names.

Syntax errors or missing file trouble generates an ERRORLEVEL 4.
CMOSChk generates an ERRORLEVEL 1 if the CMOS has changed since
the CMOSSave was done.

Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#101 - 2536 Wark Street
Victoria, BC Canada V8T 4G8
tel:(250) 361-9093
mailto:roedyg@mindprod.com
http://mindprod.com

Version History
***************

Version 1.0
- released to BIX 91/09/07

Version 1.1
- released to BIX 91/09/18
- added special check for small 64 character CMOSes.

Version 1.2
- fix spelling errors
- use CMOS.SAV instead of MyCMOS.SAV in examples

Version 1.3
- now consider fewer bytes volatile, restore more stuff.
- hints in docs about clearing CMOS.

Version 1.4
- now consider flags Status register C, offset 0C as volatile.
  fixes false alarms.

Version 1.5 1994 June 1
- change of address and phone number
- address now appears in the banner.

Version 1.6 1994 August 29
- more information about how CMOS bits are used.

Version 2.0 1994 October 1
- easier configuration of CHKCMOS volatile bytes.
- smarter bat files.
- docs on different ways to use.

Version 2.2 1995 April 16
- docs on use before experimentation

Version 2.3 1995 July 24
- add 3F as volatile byte.  PCI Pentiums are volatile.

Version 2.4 1995 August 7
- add /Q switch to suppress banners, only complain
- presume installation done to C:\CMP and that files saved there too.

Version 2.5 1996 January 27
- better documentation

Version 2.6 1996 June 3
- better documentation. Win 95 explanations.

Version 2.7 1996 August 19
- treat hex 3F differently in CMOSRest and CMOSChk

Version 2.8 1996 October  23
- POB in Quathiaski address.  Back to C:\SAFE as presumed directory
- lower cost $10, combined site licence.

Version 2.9 1996 November 22
- fix missing com files.

Version 3.0 1997 July 8
- notes on Y2K checking
- notes on separate rescue diskettes.

Version 3.1 1998 Oct 23
- new address
- Win98 ok note.

Version 3.2 1998 November 7
- slight changes to visuals to conform with standard.
- embed Barker address

Version 3.3 1999 October 4
- document problems with NT
- CMOSREST now does a read to verify that the restore succeeded.

Version 3.4 2000 May 10
- also treat byte 38 as volatile.

Version 3.5 2001 Jan 5
- also treat byte 30 as volatile at request of Sony.

Version 3.6 2001 Jan 8
- major simplification to bat files.

Version 3.7 2001 Jan 8
- treat 0A as volatile for cmoschk.

Version 3.8 2002 Feb 15
- new mailing address

Version 3.9 2006-02-05
- different pad directories
- explanation of extended CMOS

Version 4.0 2008-01-02
- add PAD, icon, renaming.

Version 4.1 2008-09-21
- clarify use of standand and extended CMOS in docs.

Version 4.3 2009-03-10
- provide ANT build
  refactor into three separate programs with common includes.
  remove all BAT files.
  remove references to BAT files and automatic mode from documentation.
  more information on use with Vista/XP
 
Version 4.4 2009-08-05
- make offsets 40, 41 and 42 volatile

Version 4.5 2009-08-11
- fix bug in cmoschk that was making it misidentify volatile bytes. Introduced by recent code reordering.

How to Assemble
***************

Use the ANT script or

to assemble with MASM 6.0 use:

ml.exe /AT /c /Fl /Zf /Zm cmossave.asm
link.exe /TINY /MAP cmossave.obj,cmossave.com,cmossave.map;

to assemble with OPTASM use:
optasm   cmossave.asm,cmossave.obj,cmossave.lst/L/N/G/S
olink    cmossave.obj,cmossave.com,/MAP/TINY;

Register Conventions
********************

Subroutines may trash all registers except those explicity
documented as input or output.

| ; end of comment

;==============================================================

stack   segment stack           ; keep MS link happy by providing null stack
stack   ends

CODE    SEGMENT PARA            ; start off in code.

;==============================================================

data    segment byte            ; provide a separate DATA segment
                                ; actually all come after the code
;==============================================================
;  V A R I A B L E S

BannerMsg   label   byte
         DB ' CMOSSave 4.5 ۲',13d,10d
         DB 13d,10d
         DB 'Saves contents of CMOS to a file on hard disk or floppy.',13,10
         DB 'copyright (c) 1991-2009 Roedy Green, Canadian Mind Products',13,10
         DB '#101 - 2536 Wark Street, Victoria, BC Canada V8T 4G8',13,10
         DB 'tel:(250) 361-9093   mailto:roedyg@mindprod.com   http://mindprod.com',13,10
         DB 'Freeware to freely distribute and use for any purpose except military.',13,10
         DB 13,10
         DB '$'

UsageMsg DB ' Error ۲',7,13,10
         DB 'Insert a formatted diskette.',13,10
         DB 'then try:',13,10
         DB 'CMOSSave A:\CMOS.Sav',13,10
         DB 'or if want to save on hard disk try:',13,10
         DB 'CMOSSave C:\SAFE\CMOS.Sav',13,10
         DB 'CMOSSave is a DOS utility, and thus can only use short 8.3 file names',13,10
         DB 'Read CMOS.TXT to find how to use it properly.',13,10
         DB '$'

FileTroubleMsg  DB ' Error ۲',7,13,10
                DB 'Cannot create the disk file.',13,10
                db '$'

WorkedMsg       DB 'CMOS successfully saved',13,10
                db '$'

ParmIndex       DW      0

Filename        DB      64 dup (0)
                        ; asciiz filename

SlashQuiet      DB      0
                        ; -1 means quiet mode /Q

CMOSSize        DB      0
                        ; size of cmos in bytes

CMOSBuff        DB  128 dup(0) ; buffer to hold CMOS contents.

data            ends

com     group   code,data       ; force data segment to go at the end

        ASSUME  CS:com,DS:com,ES:com,SS:com
                                ; seg regs cover everything
        ORG     100H            ; in Code segment

;==========================

Main    proc    far

;       M A I N L I N E   R O U T I N E
Start:  cld
        Call    Parse           ; get filename and /Q from command line

        lea     dx,BannerMsg    ; display the banner if no /Q
        Call    SayQ

        call    GetCMOS         ; fetch CMOS to buffer

        call    WriteCMOSToFile ; write CMOS contents to file
                                ; we save all CMOS bytes, even the volatile ones.

        lea     dx,WorkedMsg    ; crow about success
        Call    Say

Done:
        mov     ax,4c00h
        int     21h             ;normal termination

Main    EndP

;===============================================================

GetCMOS Proc    Near

;       Get 128 byte contents of CMOS into a buffer.

        mov     cx,128          ; count of times through loop
        lea     bx,CMOSBuff     ; where to put the contents
        sub     al,al           ; start offset in CMOS
GetLoop:
        Call    PeekCmos        ; al=offset ah=contents
        mov     byte ptr[bx],ah
        inc     al
        inc     bx
        loop    GetLoop
        ret

GetCMOS EndP

;===============================================================

WriteCMOSToFile       Proc    Near

;       Create a file; write CMOS to it

        lea     dx,FileName     ; DS:DX point to file
        xor     cx,cx           ; CX=0 is attribute
        mov     ah,03ch         ; DOS create function
        int     21h
        jc      FileTrouble
        mov     bx,ax           ; SAVE HANDLE
        MOV     CX,128          ; write 128 bytes
        lea     dx,CMOSBuff     ; buffer address
        mov     ah,40h          ; DOS write
        int     21h
        jc      FileTrouble
        cmp     ax,128
        jne     FileTrouble
        mov     ah,3eh          ; DOS close
        int     21h
        jc      FileTrouble
        ret

WriteCMOSToFile       EndP

;===============================================================

include cmosinc0.asm

;===============================================================

CODE    ends                    ; end of code segment
        end     Start
